@@ -1,6 +1,5 @@
 void hash_init (void);
 void forget_all (void);
 void forget_created (ino_t ino, dev_t dev);
-char *remember_copied (const char *node, ino_t ino, dev_t dev);
-bool remember_created (char const *file);
+char *remember_copied (char const *node, ino_t ino, dev_t dev);
 char *src_to_dest_lookup (ino_t ino, dev_t dev);
