@@ -2,67 +2,42 @@ static char const G_line[] =
 {
   '#',' ','C','o','n','f','i','g','u','r','a','t','i','o','n',' ','f','i','l','e',' ','f','o','r',' ','d','i','r','c','o','l','o','r','s',',',' ','a',' ','u','t','i','l','i','t','y',' ','t','o',' ','h','e','l','p',' ','y','o','u',' ','s','e','t',' ','t','h','e',0,
   '#',' ','L','S','_','C','O','L','O','R','S',' ','e','n','v','i','r','o','n','m','e','n','t',' ','v','a','r','i','a','b','l','e',' ','u','s','e','d',' ','b','y',' ','G','N','U',' ','l','s',' ','w','i','t','h',' ','t','h','e',' ','-','-','c','o','l','o','r',' ','o','p','t','i','o','n','.',0,
-  '#',' ','C','o','p','y','r','i','g','h','t',' ','(','C',')',' ','1','9','9','6','-','2','0','1','3',' ','F','r','e','e',' ','S','o','f','t','w','a','r','e',' ','F','o','u','n','d','a','t','i','o','n',',',' ','I','n','c','.',0,
+  '#',' ','C','o','p','y','r','i','g','h','t',' ','(','C',')',' ','1','9','9','6','-','2','0','2','1',' ','F','r','e','e',' ','S','o','f','t','w','a','r','e',' ','F','o','u','n','d','a','t','i','o','n',',',' ','I','n','c','.',0,
   '#',' ','C','o','p','y','i','n','g',' ','a','n','d',' ','d','i','s','t','r','i','b','u','t','i','o','n',' ','o','f',' ','t','h','i','s',' ','f','i','l','e',',',' ','w','i','t','h',' ','o','r',' ','w','i','t','h','o','u','t',' ','m','o','d','i','f','i','c','a','t','i','o','n',',',0,
   '#',' ','a','r','e',' ','p','e','r','m','i','t','t','e','d',' ','p','r','o','v','i','d','e','d',' ','t','h','e',' ','c','o','p','y','r','i','g','h','t',' ','n','o','t','i','c','e',' ','a','n','d',' ','t','h','i','s',' ','n','o','t','i','c','e',' ','a','r','e',' ','p','r','e','s','e','r','v','e','d','.',0,
   '#',' ','T','h','e',' ','k','e','y','w','o','r','d','s',' ','C','O','L','O','R',',',' ','O','P','T','I','O','N','S',',',' ','a','n','d',' ','E','I','G','H','T','B','I','T',' ','(','h','o','n','o','r','e','d',' ','b','y',' ','t','h','e',0,
   '#',' ','s','l','a','c','k','w','a','r','e',' ','v','e','r','s','i','o','n',' ','o','f',' ','d','i','r','c','o','l','o','r','s',')',' ','a','r','e',' ','r','e','c','o','g','n','i','z','e','d',' ','b','u','t',' ','i','g','n','o','r','e','d','.',0,
-  '#',' ','B','e','l','o','w',',',' ','t','h','e','r','e',' ','s','h','o','u','l','d',' ','b','e',' ','o','n','e',' ','T','E','R','M',' ','e','n','t','r','y',' ','f','o','r',' ','e','a','c','h',' ','t','e','r','m','t','y','p','e',' ','t','h','a','t',' ','i','s',' ','c','o','l','o','r','i','z','a','b','l','e',0,
+  '#',' ','B','e','l','o','w',' ','a','r','e',' ','T','E','R','M',' ','e','n','t','r','i','e','s',',',' ','w','h','i','c','h',' ','c','a','n',' ','b','e',' ','a',' ','g','l','o','b',' ','p','a','t','t','e','r','n','s',',',' ','t','o',' ','m','a','t','c','h',0,
+  '#',' ','a','g','a','i','n','s','t',' ','t','h','e',' ','T','E','R','M',' ','e','n','v','i','r','o','n','m','e','n','t',' ','v','a','r','i','a','b','l','e',' ','t','o',' ','d','e','t','e','r','m','i','n','e',' ','i','f',' ','i','t',' ','i','s',' ','c','o','l','o','r','i','z','a','b','l','e','.',0,
   'T','E','R','M',' ','E','t','e','r','m',0,
   'T','E','R','M',' ','a','n','s','i',0,
-  'T','E','R','M',' ','c','o','l','o','r','-','x','t','e','r','m',0,
-  'T','E','R','M',' ','c','o','n','1','3','2','x','2','5',0,
-  'T','E','R','M',' ','c','o','n','1','3','2','x','3','0',0,
-  'T','E','R','M',' ','c','o','n','1','3','2','x','4','3',0,
-  'T','E','R','M',' ','c','o','n','1','3','2','x','6','0',0,
-  'T','E','R','M',' ','c','o','n','8','0','x','2','5',0,
-  'T','E','R','M',' ','c','o','n','8','0','x','2','8',0,
-  'T','E','R','M',' ','c','o','n','8','0','x','3','0',0,
-  'T','E','R','M',' ','c','o','n','8','0','x','4','3',0,
-  'T','E','R','M',' ','c','o','n','8','0','x','5','0',0,
-  'T','E','R','M',' ','c','o','n','8','0','x','6','0',0,
+  'T','E','R','M',' ','*','c','o','l','o','r','*',0,
+  'T','E','R','M',' ','c','o','n','[','0','-','9',']','*','x','[','0','-','9',']','*',0,
   'T','E','R','M',' ','c','o','n','s','2','5',0,
   'T','E','R','M',' ','c','o','n','s','o','l','e',0,
   'T','E','R','M',' ','c','y','g','w','i','n',0,
+  'T','E','R','M',' ','*','d','i','r','e','c','t','*',0,
   'T','E','R','M',' ','d','t','t','e','r','m',0,
-  'T','E','R','M',' ','e','t','e','r','m','-','c','o','l','o','r',0,
   'T','E','R','M',' ','g','n','o','m','e',0,
-  'T','E','R','M',' ','g','n','o','m','e','-','2','5','6','c','o','l','o','r',0,
+  'T','E','R','M',' ','h','u','r','d',0,
   'T','E','R','M',' ','j','f','b','t','e','r','m',0,
   'T','E','R','M',' ','k','o','n','s','o','l','e',0,
   'T','E','R','M',' ','k','t','e','r','m',0,
   'T','E','R','M',' ','l','i','n','u','x',0,
   'T','E','R','M',' ','l','i','n','u','x','-','c',0,
-  'T','E','R','M',' ','m','a','c','h','-','c','o','l','o','r',0,
   'T','E','R','M',' ','m','l','t','e','r','m',0,
   'T','E','R','M',' ','p','u','t','t','y',0,
-  'T','E','R','M',' ','r','x','v','t',0,
-  'T','E','R','M',' ','r','x','v','t','-','2','5','6','c','o','l','o','r',0,
-  'T','E','R','M',' ','r','x','v','t','-','c','y','g','w','i','n',0,
-  'T','E','R','M',' ','r','x','v','t','-','c','y','g','w','i','n','-','n','a','t','i','v','e',0,
-  'T','E','R','M',' ','r','x','v','t','-','u','n','i','c','o','d','e',0,
-  'T','E','R','M',' ','r','x','v','t','-','u','n','i','c','o','d','e','-','2','5','6','c','o','l','o','r',0,
-  'T','E','R','M',' ','r','x','v','t','-','u','n','i','c','o','d','e','2','5','6',0,
-  'T','E','R','M',' ','s','c','r','e','e','n',0,
-  'T','E','R','M',' ','s','c','r','e','e','n','-','2','5','6','c','o','l','o','r',0,
-  'T','E','R','M',' ','s','c','r','e','e','n','-','2','5','6','c','o','l','o','r','-','b','c','e',0,
-  'T','E','R','M',' ','s','c','r','e','e','n','-','b','c','e',0,
-  'T','E','R','M',' ','s','c','r','e','e','n','-','w',0,
-  'T','E','R','M',' ','s','c','r','e','e','n','.','E','t','e','r','m',0,
-  'T','E','R','M',' ','s','c','r','e','e','n','.','r','x','v','t',0,
-  'T','E','R','M',' ','s','c','r','e','e','n','.','l','i','n','u','x',0,
+  'T','E','R','M',' ','r','x','v','t','*',0,
+  'T','E','R','M',' ','s','c','r','e','e','n','*',0,
   'T','E','R','M',' ','s','t',0,
-  'T','E','R','M',' ','s','t','-','2','5','6','c','o','l','o','r',0,
   'T','E','R','M',' ','t','e','r','m','i','n','a','t','o','r',0,
+  'T','E','R','M',' ','t','m','u','x','*',0,
   'T','E','R','M',' ','v','t','1','0','0',0,
-  'T','E','R','M',' ','x','t','e','r','m',0,
-  'T','E','R','M',' ','x','t','e','r','m','-','1','6','c','o','l','o','r',0,
-  'T','E','R','M',' ','x','t','e','r','m','-','2','5','6','c','o','l','o','r',0,
-  'T','E','R','M',' ','x','t','e','r','m','-','8','8','c','o','l','o','r',0,
-  'T','E','R','M',' ','x','t','e','r','m','-','c','o','l','o','r',0,
-  'T','E','R','M',' ','x','t','e','r','m','-','d','e','b','i','a','n',0,
-  '#',' ','B','e','l','o','w',' ','a','r','e',' ','t','h','e',' ','c','o','l','o','r',' ','i','n','i','t',' ','s','t','r','i','n','g','s',' ','f','o','r',' ','t','h','e',' ','b','a','s','i','c',' ','f','i','l','e',' ','t','y','p','e','s','.',' ','A',' ','c','o','l','o','r',' ','i','n','i','t',0,
-  '#',' ','s','t','r','i','n','g',' ','c','o','n','s','i','s','t','s',' ','o','f',' ','o','n','e',' ','o','r',' ','m','o','r','e',' ','o','f',' ','t','h','e',' ','f','o','l','l','o','w','i','n','g',' ','n','u','m','e','r','i','c',' ','c','o','d','e','s',':',0,
+  'T','E','R','M',' ','x','t','e','r','m','*',0,
+  '#',' ','B','e','l','o','w',' ','a','r','e',' ','t','h','e',' ','c','o','l','o','r',' ','i','n','i','t',' ','s','t','r','i','n','g','s',' ','f','o','r',' ','t','h','e',' ','b','a','s','i','c',' ','f','i','l','e',' ','t','y','p','e','s','.',0,
+  '#',' ','O','n','e',' ','c','a','n',' ','u','s','e',' ','c','o','d','e','s',' ','f','o','r',' ','2','5','6',' ','o','r',' ','m','o','r','e',' ','c','o','l','o','r','s',' ','s','u','p','p','o','r','t','e','d',' ','b','y',' ','m','o','d','e','r','n',' ','t','e','r','m','i','n','a','l','s','.',0,
+  '#',' ','T','h','e',' ','d','e','f','a','u','l','t',' ','c','o','l','o','r',' ','c','o','d','e','s',' ','u','s','e',' ','t','h','e',' ','c','a','p','a','b','i','l','i','t','i','e','s',' ','o','f',' ','a','n',' ','8',' ','c','o','l','o','r',' ','t','e','r','m','i','n','a','l',0,
+  '#',' ','w','i','t','h',' ','s','o','m','e',' ','a','d','d','i','t','i','o','n','a','l',' ','a','t','t','r','i','b','u','t','e','s',' ','a','s',' ','p','e','r',' ','t','h','e',' ','f','o','l','l','o','w','i','n','g',' ','c','o','d','e','s',':',0,
   '#',' ','A','t','t','r','i','b','u','t','e',' ','c','o','d','e','s',':',0,
   '#',' ','0','0','=','n','o','n','e',' ','0','1','=','b','o','l','d',' ','0','4','=','u','n','d','e','r','s','c','o','r','e',' ','0','5','=','b','l','i','n','k',' ','0','7','=','r','e','v','e','r','s','e',' ','0','8','=','c','o','n','c','e','a','l','e','d',0,
   '#',' ','T','e','x','t',' ','c','o','l','o','r',' ','c','o','d','e','s',':',0,
@@ -81,7 +56,8 @@ static char const G_line[] =
   'D','O','O','R',' ','0','1',';','3','5',' ','#',' ','d','o','o','r',0,
   'B','L','K',' ','4','0',';','3','3',';','0','1',' ','#',' ','b','l','o','c','k',' ','d','e','v','i','c','e',' ','d','r','i','v','e','r',0,
   'C','H','R',' ','4','0',';','3','3',';','0','1',' ','#',' ','c','h','a','r','a','c','t','e','r',' ','d','e','v','i','c','e',' ','d','r','i','v','e','r',0,
-  'O','R','P','H','A','N',' ','4','0',';','3','1',';','0','1',' ','#',' ','s','y','m','l','i','n','k',' ','t','o',' ','n','o','n','e','x','i','s','t','e','n','t',' ','f','i','l','e',',',' ','o','r',' ','n','o','n','-','s','t','a','t','\'','a','b','l','e',' ','f','i','l','e',0,
+  'O','R','P','H','A','N',' ','4','0',';','3','1',';','0','1',' ','#',' ','s','y','m','l','i','n','k',' ','t','o',' ','n','o','n','e','x','i','s','t','e','n','t',' ','f','i','l','e',',',' ','o','r',' ','n','o','n','-','s','t','a','t','\'','a','b','l','e',' ','f','i','l','e',' ','.','.','.',0,
+  'M','I','S','S','I','N','G',' ','0','0',' ','#',' ','.','.','.',' ','a','n','d',' ','t','h','e',' ','f','i','l','e','s',' ','t','h','e','y',' ','p','o','i','n','t',' ','t','o',0,
   'S','E','T','U','I','D',' ','3','7',';','4','1',' ','#',' ','f','i','l','e',' ','t','h','a','t',' ','i','s',' ','s','e','t','u','i','d',' ','(','u','+','s',')',0,
   'S','E','T','G','I','D',' ','3','0',';','4','3',' ','#',' ','f','i','l','e',' ','t','h','a','t',' ','i','s',' ','s','e','t','g','i','d',' ','(','g','+','s',')',0,
   'C','A','P','A','B','I','L','I','T','Y',' ','3','0',';','4','1',' ','#',' ','f','i','l','e',' ','w','i','t','h',' ','c','a','p','a','b','i','l','i','t','y',0,
@@ -106,19 +82,27 @@ static char const G_line[] =
   ' ','#',' ','a','r','c','h','i','v','e','s',' ','o','r',' ','c','o','m','p','r','e','s','s','e','d',' ','(','b','r','i','g','h','t',' ','r','e','d',')',0,
   '.','t','a','r',' ','0','1',';','3','1',0,
   '.','t','g','z',' ','0','1',';','3','1',0,
+  '.','a','r','c',' ','0','1',';','3','1',0,
   '.','a','r','j',' ','0','1',';','3','1',0,
   '.','t','a','z',' ','0','1',';','3','1',0,
+  '.','l','h','a',' ','0','1',';','3','1',0,
+  '.','l','z','4',' ','0','1',';','3','1',0,
   '.','l','z','h',' ','0','1',';','3','1',0,
   '.','l','z','m','a',' ','0','1',';','3','1',0,
   '.','t','l','z',' ','0','1',';','3','1',0,
   '.','t','x','z',' ','0','1',';','3','1',0,
+  '.','t','z','o',' ','0','1',';','3','1',0,
+  '.','t','7','z',' ','0','1',';','3','1',0,
   '.','z','i','p',' ','0','1',';','3','1',0,
   '.','z',' ','0','1',';','3','1',0,
-  '.','Z',' ','0','1',';','3','1',0,
   '.','d','z',' ','0','1',';','3','1',0,
   '.','g','z',' ','0','1',';','3','1',0,
+  '.','l','r','z',' ','0','1',';','3','1',0,
   '.','l','z',' ','0','1',';','3','1',0,
+  '.','l','z','o',' ','0','1',';','3','1',0,
   '.','x','z',' ','0','1',';','3','1',0,
+  '.','z','s','t',' ','0','1',';','3','1',0,
+  '.','t','z','s','t',' ','0','1',';','3','1',0,
   '.','b','z','2',' ','0','1',';','3','1',0,
   '.','b','z',' ','0','1',';','3','1',0,
   '.','t','b','z',' ','0','1',';','3','1',0,
@@ -131,14 +115,22 @@ static char const G_line[] =
   '.','e','a','r',' ','0','1',';','3','1',0,
   '.','s','a','r',' ','0','1',';','3','1',0,
   '.','r','a','r',' ','0','1',';','3','1',0,
+  '.','a','l','z',' ','0','1',';','3','1',0,
   '.','a','c','e',' ','0','1',';','3','1',0,
   '.','z','o','o',' ','0','1',';','3','1',0,
   '.','c','p','i','o',' ','0','1',';','3','1',0,
   '.','7','z',' ','0','1',';','3','1',0,
   '.','r','z',' ','0','1',';','3','1',0,
+  '.','c','a','b',' ','0','1',';','3','1',0,
+  '.','w','i','m',' ','0','1',';','3','1',0,
+  '.','s','w','m',' ','0','1',';','3','1',0,
+  '.','d','w','m',' ','0','1',';','3','1',0,
+  '.','e','s','d',' ','0','1',';','3','1',0,
   '#',' ','i','m','a','g','e',' ','f','o','r','m','a','t','s',0,
   '.','j','p','g',' ','0','1',';','3','5',0,
   '.','j','p','e','g',' ','0','1',';','3','5',0,
+  '.','m','j','p','g',' ','0','1',';','3','5',0,
+  '.','m','j','p','e','g',' ','0','1',';','3','5',0,
   '.','g','i','f',' ','0','1',';','3','5',0,
   '.','b','m','p',' ','0','1',';','3','5',0,
   '.','p','b','m',' ','0','1',';','3','5',0,
@@ -160,6 +152,7 @@ static char const G_line[] =
   '.','m','2','v',' ','0','1',';','3','5',0,
   '.','m','k','v',' ','0','1',';','3','5',0,
   '.','w','e','b','m',' ','0','1',';','3','5',0,
+  '.','w','e','b','p',' ','0','1',';','3','5',0,
   '.','o','g','m',' ','0','1',';','3','5',0,
   '.','m','p','4',' ','0','1',';','3','5',0,
   '.','m','4','v',' ','0','1',';','3','5',0,
@@ -182,15 +175,14 @@ static char const G_line[] =
   '.','y','u','v',' ','0','1',';','3','5',0,
   '.','c','g','m',' ','0','1',';','3','5',0,
   '.','e','m','f',' ','0','1',';','3','5',0,
-  '#',' ','h','t','t','p',':','/','/','w','i','k','i','.','x','i','p','h','.','o','r','g','/','i','n','d','e','x','.','p','h','p','/','M','I','M','E','_','T','y','p','e','s','_','a','n','d','_','F','i','l','e','_','E','x','t','e','n','s','i','o','n','s',0,
-  '.','a','x','v',' ','0','1',';','3','5',0,
-  '.','a','n','x',' ','0','1',';','3','5',0,
+  '#',' ','h','t','t','p','s',':','/','/','w','i','k','i','.','x','i','p','h','.','o','r','g','/','M','I','M','E','_','T','y','p','e','s','_','a','n','d','_','F','i','l','e','_','E','x','t','e','n','s','i','o','n','s',0,
   '.','o','g','v',' ','0','1',';','3','5',0,
   '.','o','g','x',' ','0','1',';','3','5',0,
   '#',' ','a','u','d','i','o',' ','f','o','r','m','a','t','s',0,
   '.','a','a','c',' ','0','0',';','3','6',0,
   '.','a','u',' ','0','0',';','3','6',0,
   '.','f','l','a','c',' ','0','0',';','3','6',0,
+  '.','m','4','a',' ','0','0',';','3','6',0,
   '.','m','i','d',' ','0','0',';','3','6',0,
   '.','m','i','d','i',' ','0','0',';','3','6',0,
   '.','m','k','a',' ','0','0',';','3','6',0,
@@ -199,9 +191,9 @@ static char const G_line[] =
   '.','o','g','g',' ','0','0',';','3','6',0,
   '.','r','a',' ','0','0',';','3','6',0,
   '.','w','a','v',' ','0','0',';','3','6',0,
-  '#',' ','h','t','t','p',':','/','/','w','i','k','i','.','x','i','p','h','.','o','r','g','/','i','n','d','e','x','.','p','h','p','/','M','I','M','E','_','T','y','p','e','s','_','a','n','d','_','F','i','l','e','_','E','x','t','e','n','s','i','o','n','s',0,
-  '.','a','x','a',' ','0','0',';','3','6',0,
+  '#',' ','h','t','t','p','s',':','/','/','w','i','k','i','.','x','i','p','h','.','o','r','g','/','M','I','M','E','_','T','y','p','e','s','_','a','n','d','_','F','i','l','e','_','E','x','t','e','n','s','i','o','n','s',0,
   '.','o','g','a',' ','0','0',';','3','6',0,
+  '.','o','p','u','s',' ','0','0',';','3','6',0,
   '.','s','p','x',' ','0','0',';','3','6',0,
   '.','x','s','p','f',' ','0','0',';','3','6',0,
 };
