@@ -1,6 +1,6 @@
 /* Remove directory entries.
 
-   Copyright (C) 1998-2013 Free Software Foundation, Inc.
+   Copyright (C) 1998-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef REMOVE_H
 # define REMOVE_H
@@ -37,7 +37,7 @@ struct rm_options
   /* If true, query the user about whether to remove each file.  */
   enum rm_interactive interactive;
 
-  // FIXME: remove
+  /* FIXME: remove  */
   /* If true, do not traverse into (or remove) any directory that is
      on a file system (i.e., that has a different device number) other
      than that of the corresponding command line argument.  Note that
@@ -56,6 +56,10 @@ struct rm_options
      and preserving '/'.  Otherwise NULL.  */
   struct dev_ino *root_dev_ino;
 
+  /* If true, do not traverse into (or remove) any directory that is
+     the root of a file system.  I.e., a separate device.  */
+  bool preserve_all_root;
+
   /* If nonzero, stdin is a tty.  */
   bool stdin_tty;
 
