@@ -1,5 +1,5 @@
 /* 'rm' file deletion utility for GNU.
-   Copyright (C) 1988-2013 Free Software Foundation, Inc.
+   Copyright (C) 1988-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Initially written by Paul Rubin, David MacKenzie, and Richard Stallman.
    Reworked to use chdir and avoid recursion, and later, rewritten
@@ -26,9 +26,8 @@
 
 #include "system.h"
 #include "argmatch.h"
+#include "die.h"
 #include "error.h"
-#include "quote.h"
-#include "quotearg.h"
 #include "remove.h"
 #include "root-dev-ino.h"
 #include "yesno.h"
@@ -68,7 +67,7 @@ static struct option const long_opts[] =
 
   {"one-file-system", no_argument, NULL, ONE_FILE_SYSTEM},
   {"no-preserve-root", no_argument, NULL, NO_PRESERVE_ROOT},
-  {"preserve-root", no_argument, NULL, PRESERVE_ROOT},
+  {"preserve-root", optional_argument, NULL, PRESERVE_ROOT},
 
   /* This is solely for testing.  Do not document.  */
   /* It is relatively difficult to ensure that there is a tty on stdin.
@@ -106,9 +105,8 @@ diagnose_leading_hyphen (int argc, char **argv)
 {
   /* OPTIND is unreliable, so iterate through the arguments looking
      for a file name that looks like an option.  */
-  int i;
 
-  for (i = 1; i < argc; i++)
+  for (int i = 1; i < argc; i++)
     {
       char const *arg = argv[i];
       struct stat st;
@@ -118,8 +116,8 @@ diagnose_leading_hyphen (int argc, char **argv)
           fprintf (stderr,
                    _("Try '%s ./%s' to remove the file %s.\n"),
                    argv[0],
-                   quotearg_n_style (1, shell_quoting_style, arg),
-                   quote (arg));
+                   quotearg_n_style (1, shell_escape_quoting_style, arg),
+                   quoteaf (arg));
           break;
         }
     }
@@ -132,7 +130,7 @@ usage (int status)
     emit_try_help ();
   else
     {
-      printf (_("Usage: %s [OPTION]... FILE...\n"), program_name);
+      printf (_("Usage: %s [OPTION]... [FILE]...\n"), program_name);
       fputs (_("\
 Remove (unlink) the FILE(s).\n\
 \n\
@@ -141,10 +139,10 @@ Remove (unlink) the FILE(s).\n\
 "), stdout);
       fputs (_("\
   -I                    prompt once before removing more than three files, or\n\
-                          when removing recursively.  Less intrusive than -i,\n\
+                          when removing recursively; less intrusive than -i,\n\
                           while still giving protection against most mistakes\n\
       --interactive[=WHEN]  prompt according to WHEN: never, once (-I), or\n\
-                          always (-i).  Without WHEN, prompt always\n\
+                          always (-i); without WHEN, prompt always\n\
 "), stdout);
       fputs (_("\
       --one-file-system  when removing a hierarchy recursively, skip any\n\
@@ -153,7 +151,11 @@ Remove (unlink) the FILE(s).\n\
 "), stdout);
       fputs (_("\
       --no-preserve-root  do not treat '/' specially\n\
-      --preserve-root   do not remove '/' (default)\n\
+      --preserve-root[=all]  do not remove '/' (default);\n\
+                              with 'all', reject any command line argument\n\
+                              on a separate device from its parent\n\
+"), stdout);
+      fputs (_("\
   -r, -R, --recursive   remove directories and their contents recursively\n\
   -d, --dir             remove empty directories\n\
   -v, --verbose         explain what is being done\n\
@@ -178,9 +180,9 @@ use one of these commands:\n\
 \n\
 Note that if you use rm to remove a file, it might be possible to recover\n\
 some of its contents, given sufficient expertise and/or time.  For greater\n\
-assurance that the contents are truly unrecoverable, consider using shred.\n\
+assurance that the contents are truly unrecoverable, consider using shred(1).\n\
 "), stdout);
-      emit_ancillary_info ();
+      emit_ancillary_info (PROGRAM_NAME);
     }
   exit (status);
 }
@@ -194,6 +196,7 @@ rm_option_init (struct rm_options *x)
   x->remove_empty_directories = false;
   x->recursive = false;
   x->root_dev_ino = NULL;
+  x->preserve_all_root = false;
   x->stdin_tty = isatty (STDIN_FILENO);
   x->verbose = false;
 
@@ -244,7 +247,7 @@ main (int argc, char **argv)
           break;
 
         case 'I':
-          x.interactive = RMI_NEVER;
+          x.interactive = RMI_SOMETIMES;
           x.ignore_missing_files = false;
           prompt_once = true;
           break;
@@ -289,10 +292,24 @@ main (int argc, char **argv)
           break;
 
         case NO_PRESERVE_ROOT:
+          if (! STREQ (argv[optind - 1], "--no-preserve-root"))
+            die (EXIT_FAILURE, 0,
+                 _("you may not abbreviate the --no-preserve-root option"));
           preserve_root = false;
           break;
 
         case PRESERVE_ROOT:
+          if (optarg)
+            {
+              if STREQ (optarg, "all")
+                x.preserve_all_root = true;
+              else
+                {
+                  die (EXIT_FAILURE, 0,
+                       _("unrecognized --preserve-root argument: %s"),
+                       quoteaf (optarg));
+                }
+            }
           preserve_root = true;
           break;
 
@@ -315,7 +332,7 @@ main (int argc, char **argv)
   if (argc <= optind)
     {
       if (x.ignore_missing_files)
-        exit (EXIT_SUCCESS);
+        return EXIT_SUCCESS;
       else
         {
           error (0, 0, _("missing operand"));
@@ -328,25 +345,29 @@ main (int argc, char **argv)
       static struct dev_ino dev_ino_buf;
       x.root_dev_ino = get_root_dev_ino (&dev_ino_buf);
       if (x.root_dev_ino == NULL)
-        error (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
-               quote ("/"));
+        die (EXIT_FAILURE, errno, _("failed to get attributes of %s"),
+             quoteaf ("/"));
     }
 
-  size_t n_files = argc - optind;
+  uintmax_t n_files = argc - optind;
   char **file =  argv + optind;
 
   if (prompt_once && (x.recursive || 3 < n_files))
     {
       fprintf (stderr,
                (x.recursive
-                ? _("%s: remove all arguments recursively? ")
-                : _("%s: remove all arguments? ")),
-               program_name);
+                ? ngettext ("%s: remove %"PRIuMAX" argument recursively? ",
+                            "%s: remove %"PRIuMAX" arguments recursively? ",
+                            select_plural (n_files))
+                : ngettext ("%s: remove %"PRIuMAX" argument? ",
+                            "%s: remove %"PRIuMAX" arguments? ",
+                            select_plural (n_files))),
+               program_name, n_files);
       if (!yesno ())
-        exit (EXIT_SUCCESS);
+        return EXIT_SUCCESS;
     }
 
   enum RM_status status = rm (file, &x);
   assert (VALID_STATUS (status));
-  exit (status == RM_ERROR ? EXIT_FAILURE : EXIT_SUCCESS);
+  return status == RM_ERROR ? EXIT_FAILURE : EXIT_SUCCESS;
 }
