@@ -1,6 +1,6 @@
 /* shred.c - overwrite files and devices to make it harder to recover data
 
-   Copyright (C) 1999-2013 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
    Copyright (C) 1997, 1998, 1999 Colin Plumb.
 
    This program is free software: you can redistribute it and/or modify
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    Written by Colin Plumb.  */
 
@@ -28,7 +28,7 @@
  *
  * For the theory behind this, see "Secure Deletion of Data from Magnetic
  * and Solid-State Memory", on line at
- * http://www.cs.auckland.ac.nz/~pgut001/pubs/secure_del.html
+ * https://www.cs.auckland.ac.nz/~pgut001/pubs/secure_del.html
  *
  * Just for the record, reversing one or two passes of disk overwrite
  * is not terribly difficult with hardware help.  Hook up a good-quality
@@ -45,7 +45,7 @@
  * assumption out, and the assumption that you want the data processed
  * as fast as the hard drive can spin, you can do better.
  *
- * If asked to wipe a file, this also unlinks it, renaming it to in a
+ * If asked to wipe a file, this also unlinks it, renaming it in a
  * clever way to try to leave no trace of the original filename.
  *
  * This was inspired by a desire to improve on some code titled:
@@ -80,15 +80,20 @@
 #include <assert.h>
 #include <setjmp.h>
 #include <sys/types.h>
+#if defined __linux__ && HAVE_SYS_MTIO_H
+# include <sys/mtio.h>
+#endif
 
 #include "system.h"
-#include "xstrtol.h"
+#include "argmatch.h"
+#include "xdectoint.h"
+#include "die.h"
 #include "error.h"
 #include "fcntl--.h"
 #include "human.h"
-#include "quotearg.h"		/* For quotearg_colon */
 #include "randint.h"
 #include "randread.h"
+#include "renameatu.h"
 #include "stat-size.h"
 
 /* Default number of times to overwrite.  */
@@ -104,12 +109,30 @@ enum { SECTOR_SIZE = 512 };
 enum { SECTOR_MASK = SECTOR_SIZE - 1 };
 verify (0 < SECTOR_SIZE && (SECTOR_SIZE & SECTOR_MASK) == 0);
 
+enum remove_method
+{
+  remove_none = 0,      /* the default: only wipe data.  */
+  remove_unlink,        /* don't obfuscate name, just unlink.  */
+  remove_wipe,          /* obfuscate name before unlink.  */
+  remove_wipesync       /* obfuscate name, syncing each byte, before unlink.  */
+};
+
+static char const *const remove_args[] =
+{
+  "unlink", "wipe", "wipesync", NULL
+};
+
+static enum remove_method const remove_methods[] =
+{
+  remove_unlink, remove_wipe, remove_wipesync
+};
+
 struct Options
 {
   bool force;		/* -f flag: chmod files if necessary */
   size_t n_iterations;	/* -n flag: Number of iterations */
   off_t size;		/* -s flag: size of file */
-  bool remove_file;	/* -u flag: remove file after shredding */
+  enum remove_method remove_file; /* -u flag: remove file after shredding */
   bool verbose;		/* -v flag: Print progress */
   bool exact;		/* -x flag: Do not round up file size */
   bool zero_fill;	/* -z flag: Add a final zero pass */
@@ -129,7 +152,7 @@ static struct option const long_opts[] =
   {"iterations", required_argument, NULL, 'n'},
   {"size", required_argument, NULL, 's'},
   {"random-source", required_argument, NULL, RANDOM_SOURCE_OPTION},
-  {"remove", no_argument, NULL, 'u'},
+  {"remove", optional_argument, NULL, 'u'},
   {"verbose", no_argument, NULL, 'v'},
   {"zero", no_argument, NULL, 'z'},
   {GETOPT_HELP_OPTION_DECL},
@@ -148,6 +171,10 @@ usage (int status)
       fputs (_("\
 Overwrite the specified FILE(s) repeatedly, in order to make it harder\n\
 for even very expensive hardware probing to recover the data.\n\
+"), stdout);
+      fputs (_("\
+\n\
+If FILE is -, shred standard output.\n\
 "), stdout);
 
       emit_mandatory_arg_note ();
@@ -159,7 +186,8 @@ for even very expensive hardware probing to recover the data.\n\
   -s, --size=N   shred this many bytes (suffixes like K, M, G accepted)\n\
 "), DEFAULT_PASSES);
       fputs (_("\
-  -u, --remove   truncate and remove file after overwriting\n\
+  -u             deallocate and remove file after overwriting\n\
+      --remove[=HOW]  like -u but give control on HOW to delete;  See below\n\
   -v, --verbose  show progress\n\
   -x, --exact    do not round file sizes up to the next full block;\n\
                    this is the default for non-regular files\n\
@@ -169,59 +197,46 @@ for even very expensive hardware probing to recover the data.\n\
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
       fputs (_("\
 \n\
-If FILE is -, shred standard output.\n\
-\n\
 Delete FILE(s) if --remove (-u) is specified.  The default is not to remove\n\
 the files because it is common to operate on device files like /dev/hda,\n\
-and those files usually should not be removed.  When operating on regular\n\
-files, most people use the --remove option.\n\
+and those files usually should not be removed.\n\
+The optional HOW parameter indicates how to remove a directory entry:\n\
+'unlink' => use a standard unlink call.\n\
+'wipe' => also first obfuscate bytes in the name.\n\
+'wipesync' => also sync each obfuscated byte to the device.\n\
+The default mode is 'wipesync', but note it can be expensive.\n\
 \n\
 "), stdout);
       fputs (_("\
-CAUTION: Note that shred relies on a very important assumption:\n\
-that the file system overwrites data in place.  This is the traditional\n\
-way to do things, but many modern file system designs do not satisfy this\n\
-assumption.  The following are examples of file systems on which shred is\n\
-not effective, or is not guaranteed to be effective in all file system modes:\n\
-\n\
+CAUTION: shred assumes the file system and hardware overwrite data in place.\n\
+Although this is common, many platforms operate otherwise.  Also, backups\n\
+and mirrors may contain unremovable copies that will let a shredded file\n\
+be recovered later.  See the GNU coreutils manual for details.\n\
 "), stdout);
-      fputs (_("\
-* log-structured or journaled file systems, such as those supplied with\n\
-AIX and Solaris (and JFS, ReiserFS, XFS, Ext3, etc.)\n\
-\n\
-* file systems that write redundant data and carry on even if some writes\n\
-fail, such as RAID-based file systems\n\
-\n\
-* file systems that make snapshots, such as Network Appliance's NFS server\n\
-\n\
-"), stdout);
-      fputs (_("\
-* file systems that cache in temporary locations, such as NFS\n\
-version 3 clients\n\
-\n\
-* compressed file systems\n\
-\n\
-"), stdout);
-      fputs (_("\
-In the case of ext3 file systems, the above disclaimer applies\n\
-(and shred is thus of limited effectiveness) only in data=journal mode,\n\
-which journals file data in addition to just metadata.  In both the\n\
-data=ordered (default) and data=writeback modes, shred works as usual.\n\
-Ext3 journaling modes can be changed by adding the data=something option\n\
-to the mount options for a particular file system in the /etc/fstab file,\n\
-as documented in the mount man page (man mount).\n\
-\n\
-"), stdout);
-      fputs (_("\
-In addition, file system backups and remote mirrors may contain copies\n\
-of the file that cannot be removed, and that will allow a shredded file\n\
-to be recovered later.\n\
-"), stdout);
-      emit_ancillary_info ();
+      emit_ancillary_info (PROGRAM_NAME);
     }
   exit (status);
 }
 
+/*
+ * Determine if pattern type is periodic or not.
+ */
+static bool
+periodic_pattern (int type)
+{
+  if (type <= 0)
+    return false;
+
+  unsigned char r[3];
+  unsigned int bits = type & 0xfff;
+
+  bits |= bits << 12;
+  r[0] = (bits >> 4) & 255;
+  r[1] = (bits >> 8) & 255;
+  r[2] = bits & 255;
+
+  return (r[0] != r[1]) || (r[0] != r[2]);
+}
 
 /*
  * Fill a buffer with a fixed pattern.
@@ -239,7 +254,7 @@ fillpattern (int type, unsigned char *r, size_t size)
   r[0] = (bits >> 4) & 255;
   r[1] = (bits >> 8) & 255;
   r[2] = bits & 255;
-  for (i = 3; i < size / 2; i *= 2)
+  for (i = 3; i <= size / 2; i *= 2)
     memcpy (r + i, r, i);
   if (i < size)
     memcpy (r + i, r, size - i);
@@ -331,65 +346,106 @@ direct_mode (int fd, bool enable)
     }
 
 #if HAVE_DIRECTIO && defined DIRECTIO_ON && defined DIRECTIO_OFF
-  /* This is Solaris-specific.  See the following for details:
-     http://docs.sun.com/db/doc/816-0213/6m6ne37so?q=directio&a=view  */
+  /* This is Solaris-specific.  */
   directio (fd, enable ? DIRECTIO_ON : DIRECTIO_OFF);
 #endif
 }
 
+/* Rewind FD; its status is ST.  */
+static bool
+dorewind (int fd, struct stat const *st)
+{
+  if (S_ISCHR (st->st_mode))
+    {
+#if defined __linux__ && HAVE_SYS_MTIO_H
+      /* In the Linux kernel, lseek does not work on tape devices; it
+         returns a randomish value instead.  Try the low-level tape
+         rewind operation first.  */
+      struct mtop op;
+      op.mt_op = MTREW;
+      op.mt_count = 1;
+      if (ioctl (fd, MTIOCTOP, &op) == 0)
+        return true;
+#endif
+    }
+  off_t offset = lseek (fd, 0, SEEK_SET);
+  if (0 < offset)
+    errno = EINVAL;
+  return offset == 0;
+}
+
+/* By convention, negative sizes represent unknown values.  */
+
+static bool
+known (off_t size)
+{
+  return 0 <= size;
+}
+
 /*
- * Do pass number k of n, writing "size" bytes of the given pattern "type"
- * to the file descriptor fd.   Qname, k and n are passed in only for verbose
- * progress message purposes.  If n == 0, no progress messages are printed.
+ * Do pass number K of N, writing *SIZEP bytes of the given pattern TYPE
+ * to the file descriptor FD.  K and N are passed in only for verbose
+ * progress message purposes.  If N == 0, no progress messages are printed.
  *
- * If *sizep == -1, the size is unknown, and it will be filled in as soon
- * as writing fails.
+ * If *SIZEP == -1, the size is unknown, and it will be filled in as soon
+ * as writing fails with ENOSPC.
  *
  * Return 1 on write error, -1 on other error, 0 on success.
  */
 static int
-dopass (int fd, char const *qname, off_t *sizep, int type,
-        struct randread_source *s, unsigned long int k, unsigned long int n)
+dopass (int fd, struct stat const *st, char const *qname, off_t *sizep,
+        int type, struct randread_source *s,
+        unsigned long int k, unsigned long int n)
 {
   off_t size = *sizep;
-  off_t offset;			/* Current file posiiton */
+  off_t offset;			/* Current file position */
   time_t thresh IF_LINT ( = 0);	/* Time to maybe print next status update */
   time_t now = 0;		/* Current time */
   size_t lim;			/* Amount of data to try writing */
   size_t soff;			/* Offset into buffer for next write */
   ssize_t ssize;		/* Return value from write */
 
-  /* Fill pattern buffer.  Aligning it to a 32-bit boundary speeds up randread
-     in some cases.  */
-  typedef uint32_t fill_pattern_buffer[3 * 1024];
-  union
-  {
-    fill_pattern_buffer buffer;
-    char c[sizeof (fill_pattern_buffer)];
-    unsigned char u[sizeof (fill_pattern_buffer)];
-  } r;
+  /* Fill pattern buffer.  Aligning it to a page so we can do direct I/O.  */
+  size_t page_size = getpagesize ();
+#define PERIODIC_OUTPUT_SIZE (60 * 1024)
+#define NONPERIODIC_OUTPUT_SIZE (64 * 1024)
+  verify (PERIODIC_OUTPUT_SIZE % 3 == 0);
+  size_t output_size = periodic_pattern (type)
+                       ? PERIODIC_OUTPUT_SIZE : NONPERIODIC_OUTPUT_SIZE;
+#define PAGE_ALIGN_SLOP (page_size - 1)                /* So directio works */
+#define FILLPATTERN_SIZE (((output_size + 2) / 3) * 3) /* Multiple of 3 */
+#define PATTERNBUF_SIZE (PAGE_ALIGN_SLOP + FILLPATTERN_SIZE)
+  void *fill_pattern_mem = xmalloc (PATTERNBUF_SIZE);
+  unsigned char *pbuf = ptr_align (fill_pattern_mem, page_size);
 
-  off_t sizeof_r = sizeof r;
   char pass_string[PASS_NAME_SIZE];	/* Name of current pass */
   bool write_error = false;
-  bool first_write = true;
+  bool other_error = false;
 
   /* Printable previous offset into the file */
   char previous_offset_buf[LONGEST_HUMAN_READABLE + 1];
   char const *previous_human_offset IF_LINT ( = 0);
 
-  if (lseek (fd, 0, SEEK_SET) == -1)
+  /* As a performance tweak, avoid direct I/O for small sizes,
+     as it's just a performance rather then security consideration,
+     and direct I/O can often be unsupported for small non aligned sizes.  */
+  bool try_without_directio = 0 < size && size < output_size;
+  if (! try_without_directio)
+    direct_mode (fd, true);
+
+  if (! dorewind (fd, st))
     {
       error (0, errno, _("%s: cannot rewind"), qname);
-      return -1;
+      other_error = true;
+      goto free_pattern_mem;
     }
 
   /* Constant fill patterns need only be set up once. */
   if (type >= 0)
     {
-      lim = (0 <= size && size < sizeof_r ? size : sizeof_r);
-      fillpattern (type, r.u, lim);
-      passname (r.u, pass_string);
+      lim = known (size) && size < FILLPATTERN_SIZE ? size : FILLPATTERN_SIZE;
+      fillpattern (type, pbuf, lim);
+      passname (pbuf, pass_string);
     }
   else
     {
@@ -408,8 +464,8 @@ dopass (int fd, char const *qname, off_t *sizep, int type,
   while (true)
     {
       /* How much to write this time? */
-      lim = sizeof r;
-      if (0 <= size && size - offset < sizeof_r)
+      lim = output_size;
+      if (known (size) && size - offset < output_size)
         {
           if (size < offset)
             break;
@@ -418,17 +474,20 @@ dopass (int fd, char const *qname, off_t *sizep, int type,
             break;
         }
       if (type < 0)
-        randread (s, &r, lim);
+        randread (s, pbuf, lim);
       /* Loop to retry partial writes. */
-      for (soff = 0; soff < lim; soff += ssize, first_write = false)
+      for (soff = 0; soff < lim; soff += ssize)
         {
-          ssize = write (fd, r.c + soff, lim - soff);
-          if (ssize <= 0)
+          ssize = write (fd, pbuf + soff, lim - soff);
+          if (0 < ssize)
+            assume (ssize <= lim - soff);
+          else
             {
-              if (size < 0 && (ssize == 0 || errno == ENOSPC))
+              if (! known (size) && (ssize == 0 || errno == ENOSPC))
                 {
-                  /* Ah, we have found the end of the file */
-                  *sizep = size = offset + soff;
+                  /* We have found the end of the file.  */
+                  if (soff <= OFF_T_MAX - offset)
+                    *sizep = size = offset + soff;
                   break;
                 }
               else
@@ -436,17 +495,15 @@ dopass (int fd, char const *qname, off_t *sizep, int type,
                   int errnum = errno;
                   char buf[INT_BUFSIZE_BOUND (uintmax_t)];
 
-                  /* If the first write of the first pass for a given file
-                     has just failed with EINVAL, turn off direct mode I/O
-                     and try again.  This works around a bug in Linux kernel
-                     2.4 whereby opening with O_DIRECT would succeed for some
-                     file system types (e.g., ext3), but any attempt to
-                     access a file through the resulting descriptor would
-                     fail with EINVAL.  */
-                  if (k == 1 && first_write && errno == EINVAL)
+                  /* Retry without direct I/O since this may not be supported
+                     at all on some (file) systems, or with the current size.
+                     I.e., a specified --size that is not aligned, or when
+                     dealing with slop at the end of a file with --exact.  */
+                  if (! try_without_directio && errno == EINVAL)
                     {
                       direct_mode (fd, false);
                       ssize = 0;
+                      try_without_directio = true;
                       continue;
                     }
                   error (0, errnum, _("%s: error writing at offset %s"),
@@ -455,9 +512,12 @@ dopass (int fd, char const *qname, off_t *sizep, int type,
                   /* 'shred' is often used on bad media, before throwing it
                      out.  Thus, it shouldn't give up on bad blocks.  This
                      code works because lim is always a multiple of
-                     SECTOR_SIZE, except at the end.  */
-                  verify (sizeof r % SECTOR_SIZE == 0);
-                  if (errnum == EIO && 0 <= size && (soff | SECTOR_MASK) < lim)
+                     SECTOR_SIZE, except at the end.  This size constraint
+                     also enables direct I/O on some (file) systems.  */
+                  verify (PERIODIC_OUTPUT_SIZE % SECTOR_SIZE == 0);
+                  verify (NONPERIODIC_OUTPUT_SIZE % SECTOR_SIZE == 0);
+                  if (errnum == EIO && known (size)
+                      && (soff | SECTOR_MASK) < lim)
                     {
                       size_t soff1 = (soff | SECTOR_MASK) + 1;
                       if (lseek (fd, offset + soff1, SEEK_SET) != -1)
@@ -469,17 +529,19 @@ dopass (int fd, char const *qname, off_t *sizep, int type,
                         }
                       error (0, errno, _("%s: lseek failed"), qname);
                     }
-                  return -1;
+                  other_error = true;
+                  goto free_pattern_mem;
                 }
             }
         }
 
       /* Okay, we have written "soff" bytes. */
 
-      if (offset > OFF_T_MAX - (off_t) soff)
+      if (OFF_T_MAX - offset < soff)
         {
           error (0, 0, _("%s: file too large"), qname);
-          return -1;
+          other_error = true;
+          goto free_pattern_mem;
         }
 
       offset += soff;
@@ -500,7 +562,7 @@ dopass (int fd, char const *qname, off_t *sizep, int type,
 
           if (done || !STREQ (previous_human_offset, human_offset))
             {
-              if (size < 0)
+              if (! known (size))
                 error (0, 0, _("%s: pass %lu/%lu (%s)...%s"),
                        qname, k, n, pass_string, human_offset);
               else
@@ -536,7 +598,10 @@ dopass (int fd, char const *qname, off_t *sizep, int type,
               if (dosync (fd, qname) != 0)
                 {
                   if (errno != EIO)
-                    return -1;
+                    {
+                      other_error = true;
+                      goto free_pattern_mem;
+                    }
                   write_error = true;
                 }
             }
@@ -547,18 +612,24 @@ dopass (int fd, char const *qname, off_t *sizep, int type,
   if (dosync (fd, qname) != 0)
     {
       if (errno != EIO)
-        return -1;
+        {
+          other_error = true;
+          goto free_pattern_mem;
+        }
       write_error = true;
     }
 
-  return write_error;
+free_pattern_mem:
+  free (fill_pattern_mem);
+
+  return other_error ? -1 : write_error;
 }
 
 /*
  * The passes start and end with a random pass, and the passes in between
  * are done in random order.  The idea is to deprive someone trying to
  * reverse the process of knowledge of the overwrite patterns, so they
- * have the additional step of figuring out what was done to the disk
+ * have the additional step of figuring out what was done to the device
  * before they can try to reverse or cancel it.
  *
  * First, all possible 1-bit patterns.  There are two of them.
@@ -619,7 +690,7 @@ static int const
   12, 0x111, 0x222, 0x333, 0x444, 0x666, 0x777,
   0x888, 0x999, 0xBBB, 0xCCC, 0xDDD, 0xEEE,	/* 4-bit */
   -1,				/* 1 random pass */
-        /* The following patterns have the frst bit per block flipped */
+        /* The following patterns have the first bit per block flipped */
   8, 0x1000, 0x1249, 0x1492, 0x16DB, 0x1924, 0x1B6D, 0x1DB6, 0x1FFF,
   14, 0x1111, 0x1222, 0x1333, 0x1444, 0x1555, 0x1666, 0x1777,
   0x1888, 0x1999, 0x1AAA, 0x1BBB, 0x1CCC, 0x1DDD, 0x1EEE,
@@ -683,7 +754,7 @@ genpattern (int *dest, size_t num, struct randint_source *s)
           break;
         }
       else
-        {			/* Pad out with k of the n available */
+        {			/* Pad out with n of the k available */
           do
             {
               if (n == (size_t) k || randint_choose (s, k) < n)
@@ -692,6 +763,7 @@ genpattern (int *dest, size_t num, struct randint_source *s)
                   n--;
                 }
               p++;
+              k--;
             }
           while (n);
           break;
@@ -752,13 +824,14 @@ do_wipefd (int fd, char const *qname, struct randint_source *s,
 {
   size_t i;
   struct stat st;
-  off_t size;			/* Size to write, size to read */
-  unsigned long int n;		/* Number of passes for printing purposes */
+  off_t size;		/* Size to write, size to read */
+  off_t i_size = 0;	/* For small files, initial size to overwrite inode */
+  unsigned long int n;	/* Number of passes for printing purposes */
   int *passarray;
   bool ok = true;
   struct randread_source *rs;
 
-  n = 0;		/* dopass takes n -- 0 to mean "don't print progress" */
+  n = 0;		/* dopass takes n == 0 to mean "don't print progress" */
   if (flags->verbose)
     n = flags->n_iterations + flags->zero_fill;
 
@@ -778,8 +851,11 @@ do_wipefd (int fd, char const *qname, struct randint_source *s,
       error (0, 0, _("%s: invalid file type"), qname);
       return false;
     }
-
-  direct_mode (fd, true);
+  else if (S_ISREG (st.st_mode) && st.st_size < 0)
+    {
+      error (0, 0, _("%s: file has negative size"), qname);
+      return false;
+    }
 
   /* Allocate pass array */
   passarray = xnmalloc (flags->n_iterations, sizeof *passarray);
@@ -787,19 +863,28 @@ do_wipefd (int fd, char const *qname, struct randint_source *s,
   size = flags->size;
   if (size == -1)
     {
-      /* Accept a length of zero only if it's a regular file.
-         For any other type of file, try to get the size another way.  */
       if (S_ISREG (st.st_mode))
         {
           size = st.st_size;
-          if (size < 0)
+
+          if (! flags->exact)
             {
-              error (0, 0, _("%s: file has negative size"), qname);
-              return false;
+              /* Round up to the nearest block size to clear slack space.  */
+              off_t remainder = size % ST_BLKSIZE (st);
+              if (size && size < ST_BLKSIZE (st))
+                i_size = size;
+              if (remainder != 0)
+                {
+                  off_t size_incr = ST_BLKSIZE (st) - remainder;
+                  size += MIN (size_incr, OFF_T_MAX - size);
+                }
             }
         }
       else
         {
+          /* The behavior of lseek is unspecified, but in practice if
+             it returns a positive number that's the size of this
+             device.  */
           size = lseek (fd, 0, SEEK_END);
           if (size <= 0)
             {
@@ -808,63 +893,69 @@ do_wipefd (int fd, char const *qname, struct randint_source *s,
               size = -1;
             }
         }
-
-      /* Allow 'rounding up' only for regular files.  */
-      if (0 <= size && !(flags->exact) && S_ISREG (st.st_mode))
-        {
-          size += ST_BLKSIZE (st) - 1 - (size - 1) % ST_BLKSIZE (st);
-
-          /* If in rounding up, we've just overflowed, use the maximum.  */
-          if (size < 0)
-            size = TYPE_MAXIMUM (off_t);
-        }
     }
+  else if (S_ISREG (st.st_mode)
+           && st.st_size < MIN (ST_BLKSIZE (st), size))
+    i_size = st.st_size;
 
   /* Schedule the passes in random order. */
   genpattern (passarray, flags->n_iterations, s);
 
   rs = randint_get_source (s);
 
-  /* Do the work */
-  for (i = 0; i < flags->n_iterations; i++)
+  while (true)
     {
-      int err = dopass (fd, qname, &size, passarray[i], rs, i + 1, n);
-      if (err)
+      off_t pass_size;
+      unsigned long int pn = n;
+
+      if (i_size)
         {
-          if (err < 0)
+          pass_size = i_size;
+          i_size = 0;
+          pn = 0;
+        }
+      else if (size)
+        {
+          pass_size = size;
+          size = 0;
+        }
+      /* TODO: consider handling tail packing by
+         writing the tail padding as a separate pass,
+         (that would not rewind).  */
+      else
+        break;
+
+      for (i = 0; i < flags->n_iterations + flags->zero_fill; i++)
+        {
+          int err = 0;
+          int type = i < flags->n_iterations ? passarray[i] : 0;
+
+          err = dopass (fd, &st, qname, &pass_size, type, rs, i + 1, pn);
+
+          if (err)
             {
-              memset (passarray, 0, flags->n_iterations * sizeof (int));
-              free (passarray);
-              return false;
+              ok = false;
+              if (err < 0)
+                goto wipefd_out;
             }
-          ok = false;
         }
     }
 
-  memset (passarray, 0, flags->n_iterations * sizeof (int));
-  free (passarray);
+  /* Now deallocate the data.  The effect of ftruncate is specified
+     on regular files and shared memory objects (also directories, but
+     they are not possible here); don't worry about errors reported
+     for other file types.  */
 
-  if (flags->zero_fill)
-    {
-      int err = dopass (fd, qname, &size, 0, rs, flags->n_iterations + 1, n);
-      if (err)
-        {
-          if (err < 0)
-            return false;
-          ok = false;
-        }
-    }
-
-  /* Okay, now deallocate the data.  The effect of ftruncate on
-     non-regular files is unspecified, so don't worry about any
-     errors reported for them.  */
   if (flags->remove_file && ftruncate (fd, 0) != 0
-      && S_ISREG (st.st_mode))
+      && (S_ISREG (st.st_mode) || S_TYPEISSHM (&st)))
     {
       error (0, errno, _("%s: error truncating"), qname);
-      return false;
+      ok = false;
+      goto wipefd_out;
     }
 
+wipefd_out:
+  free (passarray);
   return ok;
 }
 
@@ -926,8 +1017,8 @@ incname (char *name, size_t len)
 
 /*
  * Repeatedly rename a file with shorter and shorter names,
- * to obliterate all traces of the file name on any system that
- * adds a trailing delimiter to on-disk file names and reuses
+ * to obliterate all traces of the file name (and length) on any system
+ * that adds a trailing delimiter to on-device file names and reuses
  * the same directory slot.  Finally, unlink it.
  * The passed-in filename is modified in place to the new filename.
  * (Which is unlinked if this function succeeds, but is still present if
@@ -955,59 +1046,48 @@ wipename (char *oldname, char const *qoldname, struct Options const *flags)
 {
   char *newname = xstrdup (oldname);
   char *base = last_component (newname);
-  size_t len = base_len (base);
   char *dir = dir_name (newname);
-  char *qdir = xstrdup (quotearg_colon (dir));
+  char *qdir = xstrdup (quotef (dir));
   bool first = true;
   bool ok = true;
+  int dir_fd = -1;
 
-  int dir_fd = open (dir, O_RDONLY | O_DIRECTORY | O_NOCTTY | O_NONBLOCK);
+  if (flags->remove_file == remove_wipesync)
+    dir_fd = open (dir, O_RDONLY | O_DIRECTORY | O_NOCTTY | O_NONBLOCK);
 
   if (flags->verbose)
     error (0, 0, _("%s: removing"), qoldname);
 
-  while (len)
-    {
-      memset (base, nameset[0], len);
-      base[len] = 0;
-      do
-        {
-          struct stat st;
-          if (lstat (newname, &st) < 0)
-            {
-              if (rename (oldname, newname) == 0)
-                {
-                  if (0 <= dir_fd && dosync (dir_fd, qdir) != 0)
-                    ok = false;
-                  if (flags->verbose)
-                    {
-                      /*
-                       * People seem to understand this better than talking
-                       * about renaming oldname.  newname doesn't need
-                       * quoting because we picked it.  oldname needs to
-                       * be quoted only the first time.
-                       */
-                      char const *old = (first ? qoldname : oldname);
-                      error (0, 0, _("%s: renamed to %s"), old, newname);
-                      first = false;
-                    }
-                  memcpy (oldname + (base - newname), base, len + 1);
-                  break;
-                }
-              else
-                {
-                  /* The rename failed: give up on this length.  */
-                  break;
-                }
-            }
-          else
-            {
-              /* newname exists, so increment BASE so we use another */
-            }
-        }
-      while (incname (base, len));
-      len--;
-    }
+  if (flags->remove_file != remove_unlink)
+    for (size_t len = base_len (base); len != 0; len--)
+      {
+        memset (base, nameset[0], len);
+        base[len] = 0;
+        bool rename_ok;
+        while (! (rename_ok = (renameatu (AT_FDCWD, oldname, AT_FDCWD, newname,
+                                          RENAME_NOREPLACE)
+                               == 0))
+               && errno == EEXIST && incname (base, len))
+          continue;
+        if (rename_ok)
+          {
+            if (0 <= dir_fd && dosync (dir_fd, qdir) != 0)
+              ok = false;
+            if (flags->verbose)
+              {
+                /* People seem to understand this better than talking
+                   about renaming OLDNAME.  NEWNAME doesn't need
+                   quoting because we picked it.  OLDNAME needs to be
+                   quoted only the first time.  */
+                char const *old = first ? qoldname : oldname;
+                error (0, 0,
+                       _("%s: renamed to %s"), old, newname);
+                first = false;
+              }
+            memcpy (oldname + (base - newname), base, len + 1);
+          }
+      }
+
   if (unlink (oldname) != 0)
     {
       error (0, errno, _("%s: failed to remove"), qoldname);
@@ -1117,39 +1197,29 @@ main (int argc, char **argv)
           break;
 
         case 'n':
-          {
-            uintmax_t tmp;
-            if (xstrtoumax (optarg, NULL, 10, &tmp, NULL) != LONGINT_OK
-                || MIN (UINT32_MAX, SIZE_MAX / sizeof (int)) < tmp)
-              {
-                error (EXIT_FAILURE, 0, _("%s: invalid number of passes"),
-                       quotearg_colon (optarg));
-              }
-            flags.n_iterations = tmp;
-          }
+          flags.n_iterations = xdectoumax (optarg, 0,
+                                           MIN (ULONG_MAX,
+                                                SIZE_MAX / sizeof (int)), "",
+                                           _("invalid number of passes"), 0);
           break;
 
         case RANDOM_SOURCE_OPTION:
           if (random_source && !STREQ (random_source, optarg))
-            error (EXIT_FAILURE, 0, _("multiple random sources specified"));
+            die (EXIT_FAILURE, 0, _("multiple random sources specified"));
           random_source = optarg;
           break;
 
         case 'u':
-          flags.remove_file = true;
+          if (optarg == NULL)
+            flags.remove_file = remove_wipesync;
+          else
+            flags.remove_file = XARGMATCH ("--remove", optarg,
+                                           remove_args, remove_methods);
           break;
 
         case 's':
-          {
-            uintmax_t tmp;
-            if (xstrtoumax (optarg, NULL, 0, &tmp, "cbBkKMGTPEZY0")
-                != LONGINT_OK)
-              {
-                error (EXIT_FAILURE, 0, _("%s: invalid file size"),
-                       quotearg_colon (optarg));
-              }
-            flags.size = tmp;
-          }
+          flags.size = xnumtoumax (optarg, 0, 0, OFF_T_MAX, "cbBkKMGTPEZY0",
+                                   _("invalid file size"), 0);
           break;
 
         case 'v':
@@ -1184,12 +1254,13 @@ main (int argc, char **argv)
 
   randint_source = randint_all_new (random_source, SIZE_MAX);
   if (! randint_source)
-    error (EXIT_FAILURE, errno, "%s", quotearg_colon (random_source));
+    die (EXIT_FAILURE, errno, "%s",
+         quotef (random_source ? random_source : "getrandom"));
   atexit (clear_random_data);
 
   for (i = 0; i < n_files; i++)
     {
-      char *qname = xstrdup (quotearg_colon (file[i]));
+      char *qname = xstrdup (quotef (file[i]));
       if (STREQ (file[i], "-"))
         {
           ok &= wipefd (STDOUT_FILENO, qname, randint_source, &flags);
@@ -1202,7 +1273,7 @@ main (int argc, char **argv)
       free (qname);
     }
 
-  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
+  return ok ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 /*
  * vim:sw=2:sts=2:
