@@ -1,7 +1,7 @@
 #!/bin/sh
 # Ensure that "tail -f fifo" tails indefinitely.
 
-# Copyright (C) 2009-2013 Free Software Foundation, Inc.
+# Copyright (C) 2009-2021 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 . "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
 print_ver_ tail
@@ -24,20 +24,24 @@ mkfifo_or_skip_ fifo
 echo 1 > fifo &
 echo 1 > exp || framework_failure_
 
-timeout 10 tail -f fifo > out & pid=$!
+# Terminate any background tail process
+cleanup_() { kill $pid 2>/dev/null && wait $pid; }
 
-check_tail_output()
-{
-  local n_sec="$1"
-  test -s out || { sleep $n_sec; return 1; }
-}
+# Speedup the non inotify case
+fastpoll='-s.1 --max-unchanged-stats=1'
 
-# Wait 6.3s for tail to write something.
+timeout 10 tail $fastpoll -f fifo > out & pid=$!
+
+check_tail_output() { sleep $1; test -s out; }
+
+# Wait 12.7s for tail to write something.
 retry_delay_ check_tail_output .1 7 || fail=1
 
 compare exp out || fail=1
 
-# Kill the still-running tail, or fail if it's gone.
-kill $pid || fail=1
+# Ensure tail is still running
+kill -0 $pid || fail=1
+
+cleanup_
 
 Exit $fail
